CategoryHealthChecks = "Health Checks"
CategorySecurity = "Security"
CategoryNetworking = "Networking"
CategoryResources = "Resources"
CategoryImages = "Images"
CPURequestsLabel = "CPU requests"
CPULimitsLabel = "CPU limits"
MemoryRequestsLabel = "Memory requests"
MemoryLimitsLabel = "Memory limits"
CPURequestsFailure = "CPU requests should be set"
CPULimitsFailure = "CPU limits should be set"
MemoryRequestsFailure = "Memory requests should be set"
MemoryLimitsFailure = "Memory limits should be set"
ResourceAmountTooHighFailure = "%s should be lower than %s"
ResourceAmountTooLowFailure = "%s should be higher than %s"
ResourceAmountSuccess = "%s are within the expected range"
ResourcePresentSuccess = "%s are set"
ReadinessProbeFailure = "Readiness probe should be configured"
ReadinessProbeSuccess = "Readiness probe configured"
LivenessProbeFailure = "Liveness probe should be configured"
LivenessProbeSuccess = "Liveness probe is configured"
ImageTagFailure = "Image tag should be specified"
ImageTagSuccess = "Image tag is specified"
ImagePullPolicyFailure = "Image pull policy should be \"Always\""
ImagePullPolicySuccess = "Image pull policy is \"Always\""
HostPortFailure = "Host port should not be configured"
HostPortSuccess = "Host port is not configured"
RunAsRootFailure = "Should not be allowed to run as root"
RunAsRootSuccess = "Is not allowed to run as root"
RunAsPrivilegedFailure = "Should not be running as privileged"
RunAsPrivilegedSuccess = "Not running as privileged"
ReadOnlyFilesystemSuccess = "Filesystem is read only"
ReadOnlyFilesystemFailure = "Filesystem should be read only"
PrivilegeEscalationFailure = "Privilege escalation should not be allowed"
PrivilegeEscalationSuccess = "Privilege escalation not allowed"
SecurityCapabilitiesAddedFailure = "The following security capabilities should not be added: %v"
SecurityCapabilitiesNotDroppedSuccess = "All disallowed security capabilities have been dropped"
SecurityCapabilitiesNotDroppedFailure = "The following security capabilities should be dropped: %v"
HostAliasFailure = "Host alias should not be configured"
HostAliasSuccess = "Host alias is not configured"
HostIPCFailure = "Host IPC should not be configured"
HostIPCSuccess = "Host IPC is not configured"
HostPIDFailure = "Host PID should not be configured"
HostPIDSuccess = "Host PID is not configured"
HostNetworkFailure = "Host network should not be configured"
HostNetworkSuccess = "Host network is not configured"



from datetime import datetime
from kubernetes import client
from kubernetes.client import ApiClient

__all__ = ["NamespaceKubernetesResource", "KubernetesResourceProvider"]


class NamespaceKubernetesResource:
    def __init__(self, namespace):
        super().__init__()
        self.namespaces = namespace
        self.deployments = []
        self.stateful_sets = []
        self.daemon_sets = []
        self.jobs = []
        self.cron_jobs = []
        self.pods = []


class KubernetesResourceProvider():
    def __init__(self, host, token):
        configuration = client.Configuration()
        configuration.api_key_prefix['authorization'] = 'Bearer'
        configuration.api_key['authorization'] = token
        configuration.host = "https://{}:6443".format(host)
        configuration.verify_ssl = False
        configuration.debug = False
        self.client = ApiClient(configuration)

    def fetch_namespaced_kubernetes_resource(self, namespace):
        result = NamespaceKubernetesResource(namespace)
        result.version = self.fetch_kubernetes_version()
        result.fetch_time = datetime.now()
        result.deployments = self.fetch_deployments(namespace)
        result.stateful_sets = self.fetch_stateful_sets(namespace)
        result.daemon_sets = self.fetch_daemon_sets(namespace)
        result.jobs = self.fetch_jobs(namespace)
        result.cron_jobs = self.fetch_corn_jobs(namespace)
        result.pods = self.fetch_pods(namespace)
        return result

    def fetch_namespaces(self):
        api = client.CoreV1Api(self.client)
        return api.list_namespace().items

    def fetch_nodes(self):
        api = client.CoreV1Api(self.client)
        return api.list_node().items

    def fetch_kubernetes_version(self):
        api = client.VersionApi(self.client)
        version = "{}.{}".format(api.get_code().major, api.get_code().minor)
        return version

    def fetch_pods(self, ns=None):
        api = client.CoreV1Api(self.client)
        if not ns:
            result = api.list_pod_for_all_namespaces().items
        else:
            result = api.list_namespaced_pod(ns).items
        return result

    def fetch_corn_jobs(self, ns=None):
        api = client.BatchV1beta1Api(self.client)
        if not ns:
            result = api.list_cron_job_for_all_namespaces().items
        else:
            result = api.list_namespaced_cron_job(ns).items
        return result

    def fetch_jobs(self, ns=None):
        api = client.BatchV1Api(self.client)
        if not ns:
            result = api.list_job_for_all_namespaces().items
        else:
            result = api.list_namespaced_job(ns).items
        return result

    def fetch_daemon_sets(self, ns=None):
        api = client.AppsV1Api(self.client)
        if not ns:
            result = api.list_daemon_set_for_all_namespaces().items
        else:
            result = api.list_namespaced_daemon_set(ns).items
        return result

    def fetch_stateful_sets(self, ns=None):
        api = client.AppsV1Api(self.client)
        if not ns:
            result = api.list_stateful_set_for_all_namespaces().items
        else:
            result = api.list_namespaced_stateful_set(ns).items
        return result

    def fetch_deployments(self, ns=None):
        api = client.AppsV1Api(self.client)
        if not ns:
            result = api.list_deployment_for_all_namespaces().items
        else:
            result = api.list_namespaced_deployment(ns).items
        return result



from datetime import datetime

from validator.kube.resource import KubernetesResourceProvider
from validator.base import ClusterResult
from validator.namespace import validate_namespaces


def run_validate(host, token):
    provider = KubernetesResourceProvider(host, token)
    ns = validate_namespaces(provider)
    now = datetime.now()
    return ClusterResult(ns, now)



__all__ = ["check_list"]

__default_check_list = {
    "security": {
        "hostIPCSet": "error",
        "hostPIDSet": "error",
        "notReadOnlyRootFileSystem": "warning",
        "privilegeEscalationAllowed": "error",
        "runAsRootAllowed": "warning",
        "runAsPrivileged": "error",
    },
    "kube": {
        "cpuRequestsMissing": "warning",
        "cpuLimitsMissing": "warning",
        "memoryRequestsMissing": "warning",
        "memoryLimitsMissing": "warning",
    },
    "images": {
        "tagNotSpecified": "error",
        "pullPolicyNotAlways": "ignore"
    },
    "healthChecks": {
        "readinessProbeMissing": "warning",
        "livenessProbeMissing": "warning"
    },
    "networking": {
        "hostNetworkSet": "warning",
        "hostPortSet": "warning"
    },
    "controllers": ["Deployments", "StatefulSets", "DaemonSets", "CronJobs", "Jobs"]
}

check_list = __default_check_list



RESOURCE_VALIDATION_SUCCESS = "success"
RESOURCE_VALIDATION_ERROR = "error"
RESOURCE_VALIDATION_WARNING = "warning"


class ResourceValidation:
    def __init__(self):
        self.errors = []
        self.warnings = []
        self.successes = []

    def on_failure(self, message, severity, category, name):
        from .base import ResultMessage
        if severity == "error":
            self.errors.append(ResultMessage(name, message, RESOURCE_VALIDATION_ERROR, category))
        elif severity == "warning":
            self.warnings.append(ResultMessage(name, message, RESOURCE_VALIDATION_WARNING, category))

    def on_success(self, message, category, name):
        from .base import ResultMessage
        self.successes.append(ResultMessage(name, message, RESOURCE_VALIDATION_SUCCESS, category))

    @property
    def messages(self):
        result = []
        result.extend(self.errors)
        result.extend(self.successes)
        result.extend(self.warnings)
        return result



from validator.controllers.cron_job import CronJobController
from validator.controllers.daemon_set import DaemonSetController
from validator.controllers.deployment import DeploymentController
from validator.controllers.job import JobController
from validator.controllers.stateful_set import StateSetController



from validator.controller import Controller
from kubernetes.client.models.v1_stateful_set import V1StatefulSet

__all__ = ['StateSetController']


class StateSetController(Controller):
    def __init__(self, res: V1StatefulSet):
        self.name = res.metadata.name
        self.namespace = res.metadata.namespace
        self.resource = res

    @property
    def pod_template(self):
        return self.resource.spec.template

    @property
    def pod_spec(self):
        return self.resource.spec.template.spec

    @property
    def annotations(self):
        return self.resource.metadata.annotations

    @property
    def type(self):
        return "StateSets"



from validator.controller import Controller
from kubernetes.client.models.v1beta1_cron_job import V1beta1CronJob

__all__ = ["CronJobController"]


class CronJobController(Controller):
    def __init__(self, res: V1beta1CronJob):
        self.name = res.metadata.name
        self.namespace = res.metadata.namespace
        self.resource = res

    @property
    def pod_template(self):
        return self.resource.spec.job_template.spec.template

    @property
    def pod_spec(self):
        return self.resource.spec.job_template.spec.template.spec

    @property
    def annotations(self):
        return self.resource.metadata.annotations

    @property
    def type(self):
        return "CronJobController"



import logging

from validator.config import check_list
from validator.kube.resource import NamespaceKubernetesResource
from validator.base import ControllerResult

from validator.pod import validate_pod

log = logging.getLogger("controller")

__supported_controller = ["Deployments",
                          "StatefulSets",
                          "DaemonSets",
                          "Jobs",
                          "CronJobs"]


class Controller:
    name = None
    namespace = None

    @property
    def pod_template(self):
        return

    @property
    def pod_spec(self):
        return

    @property
    def type(self):
        return

    @property
    def annotations(self):
        return


def create_validate_controllers(res: NamespaceKubernetesResource):
    from validator.controllers import DeploymentController, DaemonSetController, JobController, CronJobController, \
        StateSetController
    ctr = []
    _config_ctr = check_list.get("controllers", [])
    for cctr in _config_ctr:
        if not cctr in __supported_controller:
            log.warning("not supported controller: {} skip it".format(cctr))
        else:
            if cctr == 'Deployments':
                for d in res.deployments:
                    ctr.append(DeploymentController(d))
            elif cctr == 'DaemonSets':
                for d in res.daemon_sets:
                    ctr.append(DaemonSetController(d))
            elif cctr == "Jobs":
                for d in res.jobs:
                    ctr.append(JobController(d))
            elif cctr == "CronJobs":
                for d in res.cron_jobs:
                    ctr.append(CronJobController(d))
            elif cctr == "StateSets":
                for d in res.stateful_sets:
                    ctr.append(StateSetController(d))
    return ctr


def validate_controller(c: Controller):
    pod_result = validate_pod(c.pod_spec)
    return ControllerResult(
        name=c.name,
        namespace=c.namespace,
        type=c.type,
        pod_result=pod_result
    )


def validate_controllers(res: NamespaceKubernetesResource):
    ctr = create_validate_controllers(res)
    controllers_result = []
    for c in ctr:
        controllers_result.append(validate_controller(c))
    return controllers_result



from __future__ import print_function
from setuptools import setup, find_packages

setup(
    name="KubeGrade",
    version="0.0.4",
    author="KubeOperator",
    author_email="support@fit2cloud.com",
    description="A python library for scoring kubernetes",
    license="MIT License",
    url="https://github.com/KubeOperator/KubeGrade",
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
        'Programming Language :: Python :: 3',
    ],
    install_requires=[
        "kubernetes>=10.0.1"
    ],
    zip_safe=True,
)



import logging
import threading

from validator.kube.resource import KubernetesResourceProvider
from .base import NamespaceResult, NamespaceInfo
from .controller import validate_controllers

__all__ = ["validate_namespaces"]

log = logging.getLogger("namespace validator")


def validate_namespace_resource(namespace, provider, results):
    namespace_resource = provider.fetch_namespaced_kubernetes_resource(namespace)
    controller_results = validate_controllers(namespace_resource)
    result = NamespaceResult(
        name=namespace,
        controller_results=controller_results
    )
    results.append(result)


def validate_namespaces(provider: KubernetesResourceProvider):
    namespaces = provider.fetch_namespaces()
    results = []
    pool = []
    for namespace in namespaces:
        namespace_name = namespace.metadata.name
        t = threading.Thread(target=validate_namespace_resource, args=(namespace_name, provider, results))
        t.start()
        pool.append(t)
        for t in pool:
            t.join()
    return results



import datetime
from typing import Dict, List
from validator.kube.resource import NamespaceKubernetesResource
import json

__all__ = ["ResultSummary", "ClusterResult", "ClusterResultJsonEncoder", "ContainerResult"]


class ResultMessage:
    def __init__(self, id, message, type, category):
        self.id = id
        self.message = message
        self.type = type
        self.category = category


def count_messages(messages: List[ResultMessage]):
    from validator.resource import RESOURCE_VALIDATION_ERROR, RESOURCE_VALIDATION_WARNING, \
        RESOURCE_VALIDATION_SUCCESS
    by_category = dict()
    for msg in messages:
        category_data = CountSummary()
        if msg.type == RESOURCE_VALIDATION_ERROR:
            category_data.errors += 1
        elif msg.type == RESOURCE_VALIDATION_WARNING:
            category_data.warnings += 1
        elif msg.type == RESOURCE_VALIDATION_SUCCESS:
            category_data.successes += 1
        if by_category.__contains__(msg.category):
            by_category[msg.category].add(category_data)
        else:
            by_category[msg.category] = category_data
    return ResultSummary(by_category)


class CountSummary:
    def __init__(self, successes=0, errors=0, warnings=0):
        self.successes = successes
        self.warnings = warnings
        self.errors = errors

    def add(self, item):
        self.successes += item.successes
        self.warnings += item.warnings
        self.errors += item.errors


class ResultSummary:

    def __init__(self, category_summary: Dict[str, CountSummary]):
        self.by_category = category_summary

    @property
    def totals(self):
        count = CountSummary()
        for k in self.by_category:
            count.add(self.by_category[k])
        return count

    def add(self, res):
        if res is None:
            return
        keys = set()
        list(self.by_category.keys()).extend(list())
        for k in self.by_category: keys.add(k)
        for k in res.by_category: keys.add(k)
        for key in keys:
            if res.by_category.__contains__(key) and self.by_category.__contains__(key):
                self.by_category[key].add(res.by_category[key])
            elif res.by_category.__contains__(key):
                self.by_category[key] = res.by_category[key]


class ContainerResult:
    def __init__(self, name, messages: List[ResultMessage]):
        self.name = name
        self.messages = messages

    @property
    def summary(self):
        return count_messages(self.messages)


class PodResult:
    def __init__(self, messages: List[ResultMessage], container_results: List[ContainerResult]):
        self.messages = messages
        self.container_results = container_results

    @property
    def summary(self):
        s = count_messages(self.messages)
        for cr in self.container_results:
            s.add(cr.summary)
        return s


class ControllerResult:
    def __init__(self, name, namespace, type, pod_result: PodResult):
        self.name = name
        self.namespace = namespace
        self.type = type
        self.pod_result = pod_result


class NamespaceInfo():
    def __init__(self, namespace_resource: NamespaceKubernetesResource):
        self.count = {
            "pods": len(namespace_resource.pods),
            "deployments": len(namespace_resource.deployments),
            "statefulSets": len(namespace_resource.stateful_sets),
            "daemonSets": len(namespace_resource.daemon_sets),
            "jobs": len(namespace_resource.jobs),
            "cronJobs": len(namespace_resource.cron_jobs),
        }


class NamespaceResult:
    def __init__(self, name, controller_results):
        self.controller_results = controller_results
        self.name = name

    @property
    def summary(self):
        s = None
        for cr in self.controller_results:
            if s is None:
                s = cr.pod_result.summary
            else:
                s.add(cr.pod_result.summary)
        return s

    def append_controller_result(self, controller_result: ControllerResult):
        self.controller_results.append(controller_result)


class ClusterResult:
    def __init__(self, namespace_results, created_time=datetime.datetime.now()):
        self.created_time = created_time
        self.namespace_results = namespace_results

    @property
    def summary(self):
        s = None
        for cr in self.namespace_results:
            if s is None:
                s = cr.summary
            else:
                s.add(cr.summary)
        return s

    @property
    def score(self):
        if self.summary:
            total = self.summary.totals.successes * 2 + self.summary.totals.warnings + self.summary.totals.errors * 2
            return (float(self.summary.totals.successes * 2) / total) * 100

    @property
    def grade(self):
        score = self.score
        if score >= 97:
            return "A+"
        elif score >= 93:
            return "A"
        elif score >= 90:
            return "A-"
        elif score >= 87:
            return "B+"
        elif score >= 83:
            return "B"
        elif score >= 80:
            return "B-"
        elif score >= 77:
            return "C+"
        elif score >= 77:
            return "C"
        elif score >= 77:
            return "C-"
        elif score >= 67:
            return "D+"
        elif score >= 63:
            return "D"
        elif score >= 60:
            return "D-"
        else:
            return "F"


class ClusterResultJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, ClusterResult):
            return {
                "namespace_results": obj.namespace_results,
                "score": obj.score,
                "summary": obj.summary,
                "grade": obj.grade
            }
        if isinstance(obj, NamespaceResult):
            return {
                "controller_results": obj.controller_results,
                "summary": obj.summary,
                "name": obj.name
            }
        if isinstance(obj, ControllerResult):
            return {
                "name": obj.name,
                "type": obj.type,
                "namespace": obj.namespace,
                "pod_result": obj.pod_result
            }

        if isinstance(obj, PodResult):
            return {
                "container_results": obj.container_results,
                "summary": obj.summary,
                "messages": obj.messages,
            }
        if isinstance(obj, ContainerResult):
            return {
                "messages": obj.messages,
                "summary": obj.summary,
                "name": obj.name
            }
        if isinstance(obj, ResultSummary):
            return {
                "totals": obj.totals,
                "by_category": obj.by_category
            }
        if isinstance(obj, ResultMessage):
            return {
                "type": obj.type,
                "category": obj.category,
                "message": obj.message,
                "id": obj.id
            }
        if isinstance(obj, CountSummary):
            return {
                "successes": obj.successes,
                "errors": obj.errors,
                "warnings": obj.warnings
            }

        if isinstance(obj, datetime.datetime):
            return obj.strftime("%Y-%m-%d %H:%M:%S")
        return json.JSONEncoder.default(self, obj)



from kubernetes.client import V1SecurityContext, V1PodSpec, V1PodSecurityContext

from validator.config import check_list
from validator.resource import ResourceValidation
from . import messages
from kubernetes.client.models.v1_container import V1Container
from .base import ContainerResult


class ContainerValidation(ResourceValidation):
    def __init__(self, container, parent):
        super().__init__()
        self.container: V1Container = container
        self.init = True
        self.parent: V1PodSpec = parent

    def validate_resources(self):
        if not self.init:
            return
        category = messages.CategoryResources
        res = self.container.resources
        resource_conf = check_list.get("kube", None)

        def validate_cpu_request(cv: ContainerValidation):
            missing_name = "cpuRequestsMissing"
            if res.requests and res.requests.__contains__("cpu"):
                cv.on_success(messages.CPURequestsLabel, category, missing_name)
            else:
                severity = resource_conf.get(missing_name, None)
                cv.on_failure(messages.CPURequestsFailure, severity, category, missing_name)

        def validate_cpu_limits(cv: ContainerValidation):
            missing_name = "cpuLimitsMissing"
            if res.limits and res.limits.__contains__("cpu"):
                cv.on_success(messages.CPULimitsLabel, category, missing_name)
            else:
                severity = resource_conf.get(missing_name, None)
                cv.on_failure(messages.CPULimitsFailure, severity, category, missing_name)

        def validate_memory_request(cv: ContainerValidation):
            missing_name = "memoryRequestsMissing"
            if res.requests and res.requests.__contains__("memory"):
                cv.on_success(messages.MemoryRequestsLabel, category, missing_name)
            else:
                severity = resource_conf.get(missing_name, None)
                cv.on_failure(messages.MemoryRequestsFailure, severity, category, missing_name)

        def validate_memory_limits(cv: ContainerValidation):
            missing_name = "memoryLimitsMissing"
            if res.limits and res.limits.__contains__("memory"):
                cv.on_success(messages.MemoryLimitsLabel, category, missing_name)
            else:
                severity = resource_conf.get(missing_name, None)
                cv.on_failure(messages.MemoryLimitsFailure, severity, category, missing_name)

        validate_cpu_request(self)
        validate_cpu_limits(self)
        validate_memory_limits(self)
        validate_memory_request(self)

    def validate_health_checks(self):
        category = messages.CategoryHealthChecks
        health_check_conf = check_list.get("healthChecks", None)

        def validate_readiness_probe(cv: ContainerValidation):
            name = "readinessProbeMissing"
            if not cv.container.readiness_probe:
                severity = health_check_conf.get(name, None)
                cv.on_failure(messages.ReadinessProbeFailure, severity, category, name)
            else:
                cv.on_success(messages.ReadinessProbeSuccess, category, name)

        def validate_liveness_probe(cv: ContainerValidation):
            name = "livenessProbeMissing"
            if not cv.container.liveness_probe:
                severity = health_check_conf.get(name, None)
                cv.on_failure(messages.LivenessProbeFailure, severity, category, name)
            else:
                cv.on_success(messages.LivenessProbeSuccess, category, name)

        validate_liveness_probe(self)
        validate_readiness_probe(self)

    def validate_images(self):
        category = messages.CategoryImages
        images_conf = check_list.get("images", None)

        def validate_pull_policy(cv: ContainerValidation):
            name = "pullPolicyNotAlways"
            severity = images_conf.get(name, None)
            if not cv.container.image_pull_policy == 'Always':
                cv.on_failure(messages.ImagePullPolicyFailure, severity, category, name)
            else:
                cv.on_success(messages.ImagePullPolicySuccess, category, name)

        def validate_tag_not_specified(cv: ContainerValidation):
            name = "tagNotSpecified"
            severity = images_conf.get(name, None)
            image = cv.container.image.split(":")
            if len(image) == 1 or image[-1] == "latest":
                cv.on_failure(messages.ImageTagFailure, severity, category, name)
            else:
                cv.on_success(messages.ImageTagSuccess, category, name)

        validate_pull_policy(self)
        validate_tag_not_specified(self)

    def validate_networking(self):
        category = messages.CategoryNetworking
        networking_conf = check_list.get("networking", None)

        def validate_host_port_set(cv: ContainerValidation):
            name = "hostPortSet"
            severity = networking_conf.get(name, None)
            host_port_set = False
            if cv.container.ports:
                for port in cv.container.ports:
                    if not port.host_port:
                        host_port_set = True
                        break
            if host_port_set:
                cv.on_failure(messages.HostPortFailure, severity, category, name)
            else:
                cv.on_success(messages.HostNetworkSuccess, category, name)

        ## validate_host_port_set(self)

    def validate_security(self):
        category = messages.CategorySecurity
        security_conf = check_list.get("security", None)
        security_context = self.container.security_context
        pod_security_context = self.parent.security_context
        if not security_context:
            security_context = V1SecurityContext()
        if not pod_security_context:
            pod_security_context = V1PodSecurityContext()

        def validate_run_as_root_allowed(cv: ContainerValidation):
            name = "runAsRootAllowed"
            severity = security_conf.get(name, None)
            success = False
            if security_context.run_as_non_root or (
                    security_context.run_as_user is not None and security_context.run_as_user > 0):
                success = True
            elif security_context.run_as_non_root is None and security_context.run_as_user is None:
                success = pod_security_context.run_as_non_root or (
                        pod_security_context.run_as_user is not None and pod_security_context.run_as_user > 0)
            if success:
                cv.on_success(messages.RunAsRootSuccess, category, name)
            else:
                cv.on_failure(messages.RunAsRootFailure, severity, category, name)

        def validate_run_as_privileged(cv: ContainerValidation):
            name = "runAsPrivileged"
            severity = security_conf.get(name, None)
            if security_context.privileged:
                cv.on_failure(messages.RunAsPrivilegedFailure, severity, category, name)
            else:
                cv.on_success(messages.RunAsPrivilegedSuccess, category, name)

        def validate_not_read_only_root_file_system(cv: ContainerValidation):
            name = "notReadOnlyRootFileSystem"
            severity = security_conf.get(name, None)
            if security_context.read_only_root_filesystem:
                cv.on_success(messages.ReadOnlyFilesystemFailure, category, name)
            else:
                cv.on_failure(messages.ReadOnlyFilesystemSuccess, severity, category, name)

        def validate_privilege_escalation_allowed(cv: ContainerValidation):
            name = "privilegeEscalationAllowed"
            severity = security_conf.get(name, None)
            if security_context.allow_privilege_escalation:
                cv.on_failure(messages.PrivilegeEscalationFailure, severity, category, name)
            else:
                cv.on_success(messages.PrivilegeEscalationSuccess, category, name)

        validate_run_as_root_allowed(self)
        validate_run_as_privileged(self)
        validate_not_read_only_root_file_system(self)
        ## validate_privilege_escalation_allowed(self)


def validate_container(con, parent):
    cv = ContainerValidation(con, parent)
    cv.validate_resources()
    cv.validate_networking()
    cv.validate_health_checks()
    cv.validate_images()
    cv.validate_security()
    return ContainerResult(con.name, cv.messages)



from .resource import ResourceValidation
from kubernetes.client.models import V1PodSpec
from validator.config import check_list
from . import messages
from .base import PodResult
from .container import validate_container


class PodValidation(ResourceValidation):

    def __init__(self, pod):
        super().__init__()
        self.pod: V1PodSpec = pod

    def validate_security(self):
        category = messages.CategorySecurity

        def validate_security_host_ipc_set(pv):
            name = "hostIPCSet"
            security_conf = check_list.get("security", None)
            severity = security_conf.get(name, None)
            if pv.pod.host_ipc:
                pv.on_failure(messages.HostIPCFailure, severity, category, name)
            else:
                pv.on_success(messages.HostIPCSuccess, category, name)

        def validate_security_host_pid_set(pv):
            name = "hostPIDSet"
            security_conf = check_list.get("security", None)
            severity = security_conf.get(name, None)
            if pv.pod.host_pid:
                pv.on_failure(messages.HostPIDFailure, severity, category, name)
            else:
                pv.on_success(messages.HostPIDSuccess, category, name)

        validate_security_host_ipc_set(self)
        ## validate_security_host_pid_set(self)

    def validate_networking(self):
        category = messages.CategoryNetworking
        name = "HostNetworkSet"
        networking_conf = check_list.get("networking", None)
        severity = networking_conf.get(name, None)
        if self.pod.host_network:
            self.on_failure(messages.HostNetworkFailure, severity, category, name)
        else:
            self.on_success(messages.HostNetworkSuccess, category, name)

    def validate_containers(self):
        containers = self.pod.containers
        container_results = []
        for container in containers:
            res = validate_container(container, self.pod)
            container_results.append(res)
        return container_results


def validate_pod(pod: V1PodSpec):
    pv = PodValidation(pod)
    pv.validate_networking()
    pv.validate_security()
    containers_result = pv.validate_containers()
    return PodResult(messages=pv.messages, container_results=containers_result)
